# -*- coding: UTF-8 -*-

import sys
import subprocess
import os
from os.path import join, getsize
import dbus
from dbus.mainloop.glib import DBusGMainLoop
import gettext
import shutil
import filecmp
import re
import threading
import time
from gi import require_version
require_version('UDisks', '2.0')
require_version('Gtk', '3.0')
from gi.repository import Gio, GLib, UDisks, GObject, Gtk, Gdk

LOCALEDIR = "/usr/share/parda"
PATH = "/usr/share/parda"
gettext.bindtextdomain('parda', LOCALEDIR)
gettext.textdomain('parda')
_ = gettext.gettext

not_interesting = (
	'/org/freedesktop/UDisks2/block_devices/dm',
	'/org/freedesktop/UDisks2/block_devices/ram',
	'/org/freedesktop/UDisks2/block_devices/zram',
	'/org/freedesktop/UDisks2/drives/',
	)

loop_prefix = '/org/freedesktop/UDisks2/block_devices/loop'

GObject.threads_init()
Gdk.threads_init()

def calcular_tamano(dir):
	try:
		total = 0
		for root, dirs, files in os.walk(dir):
			relpath = root.replace(dir,'')
			if re.search("^\/\.", relpath) != None:
				continue
			tam = 0
			for name in files:
				if os.path.isfile(join(root, name)):
					tam = getsize(join(root, name))
					total = total + tam
		return total
	except (OSError) as msg:
		pass
	return -1

def format_size(size):
	"""Format a partition size."""
	# Taken from ubiquity's ubiquity/misc.py
	if size == "":
		size = 0
	if size < 1024:
		unit = 'B'
		factor = 1
	elif size < 1024 * 1024:
		unit = 'kB'
		factor = 1024
	elif size < 1024 * 1024 * 1024:
		unit = 'MB'
		factor = 1024 * 1024
	elif size < 1024 * 1024 * 1024 * 1024:
		unit = 'GB'
		factor = 1024 * 1024 * 1024
	else:
		unit = 'TB'
		factor = 1024 * 1024 * 1024 * 1024
	return '%.1f %s' % (float(size) / factor, unit)

class parda_Backend:
	def __init__(self, frontend):
		self.frontend = frontend
		self.devices = {}
		DBusGMainLoop(set_as_default=True)
		self.bus = dbus.SystemBus()
		self.udisks = UDisks.Client.new_sync(None)

	def detectar_dispositivos(self):
		self.manager = self.udisks.get_object_manager()
		self.manager.connect('object-added', lambda man, obj: self.incluir_dispositivo(obj))
		self.manager.connect('object-removed', lambda man, obj: self.borrar_dispositivo(obj))
		
		for obj in self.manager.get_objects():
			self.incluir_dispositivo(obj)
	
	def incluir_dispositivo(self, obj):
		path = obj.get_object_path()
		for boring in not_interesting:
			if path.startswith(boring):
				return
		block = obj.get_block()
		if not block:
			return
		drive_name = block.get_cached_property('Drive').get_string()
		if drive_name != '/':
			drive = self.udisks.get_object(drive_name).get_drive()
		else:
			drive = None

		if drive and drive.get_cached_property('Optical').get_boolean():
			return

		part = obj.get_partition()
		is_system = block.get_cached_property('HintSystem').get_boolean()
		is_loop = path.startswith(loop_prefix)
		if not (is_system or is_loop):
			if not part:
				return
			else:
				self.incluir_dispositivo_udisk(obj, block, drive)

	def incluir_dispositivo_udisk(self, obj, block, drive):
		fs = obj.get_filesystem()
	
		if drive:
			vendor = drive.get_cached_property('Vendor').get_string()
			model = drive.get_cached_property('Model').get_string()
			size = block.get_cached_property('Size').get_uint64()
			try:
				mountpoint = fs.get_cached_property('MountPoints').get_bytestring_array()[0]
				stat = os.statvfs(mountpoint)
				free = stat.f_bsize * stat.f_bavail
			except (IndexError, KeyError) as e:
				mountpoint = None
				free = 0
			fstype = block.get_cached_property('IdType').get_string()
		else:
			vendor = ''
			model = ''
			size = block.get_cached_property('Size').get_uint64()
			mountpoint = fs.get_cached_property('MountPoints').get_bytestring_array()[0]
			stat = os.statvfs(mountpoint)
			free = stat.f_bsize * stat.f_bavail
			fstype = block.get_cached_property('IdType').get_string()

		label = block.get_cached_property('IdLabel').get_string()

		if size <= 0:
			return
		key = block.get_cached_property('Device').get_bytestring().decode('utf-8')
		path = obj.get_object_path()
		self.devices[path] = {
			'vendor' : vendor,
			'model' : model,
			'label' : label,
			'fstype' : fstype,
			'mountpoint' : mountpoint,
			'free' : free,
			'capacity' : size,
			'device' : block.get_cached_property('Device').get_bytestring().decode('utf-8'),
			'obj' : obj,
			'path' : path
		}
		self.frontend.actualizar_dispositivos(self.devices)


	def borrar_dispositivo(self, obj):
		path = obj.get_object_path()
		for boring in not_interesting:
			if path.startswith(boring):
				return
		block = obj.get_block()
		if not block:
			return
		drive_name = block.get_cached_property('Drive').get_string()
		if drive_name != '/':
			drive = self.udisks.get_object(drive_name).get_drive()
		else:
			drive = None

		if drive and drive.get_cached_property('Optical').get_boolean():
			return

		part = obj.get_partition()
		is_system = block.get_cached_property('HintSystem').get_boolean()
		is_loop = path.startswith(loop_prefix)
		if not (is_system or is_loop):
			try:
				self.frontend.actualizar_dispositivos(self.devices)
				self.devices.pop(path)
			except (KeyError) as e:
				pass

	def get_device_info(self, obj, path):
		try:
			fs = obj.get_filesystem()
			mountpoint = fs.get_cached_property('MountPoints').get_bytestring_array()[0]
			stat = os.statvfs(mountpoint)
			free = stat.f_bsize * stat.f_bavail
			self.devices[path]['mountpoint'] = mountpoint
			self.devices[path]['free'] = free
		except (IndexError, KeyError) as e:
			mountpoint = None
			free = 0
		return (mountpoint, free)

class mensaje_Aviso:
	def __init__(self, msg):
		self.cadena = msg

		# Se prepara el mensaje de entrada para su publicacion
		self.indice_linea = self.preparar_mensaje()
		
		# Ventana del mensaje y contenedor
		self.warning_window = Gtk.Window(Gtk.WindowType.TOPLEVEL)
		self.warning_window.set_position(Gtk.WindowPosition.CENTER)
		self.warning_window.set_title(_('Parda - Aviso'))
		self.warning_window.set_border_width(10)
		self.warning_window.set_resizable(False)
		self.warning_window.connect("delete_event", self.delete_event)
		self.warning_window.connect("destroy", self.destroy)
		self.vbox = Gtk.VBox(False, 30)
		self.panel = Gtk.Fixed()
		self.vbox.pack_start(self.panel, True, True, 0)
		self.warning_window.add(self.vbox)


		# Cabecera de la ventana de aviso
		self.imagen_aviso = Gtk.Image()
	#	self.imagen_aviso.set_from_file('%s/img/warning.png' % PATH)
		self.imagen_aviso.set_from_icon_name("dialog-warning-symbolic", 48)
		self.panel.put(self.imagen_aviso, 200, 10)
		self.et_aviso = Gtk.Label()
		self.et_aviso.set_markup(_('<b>Mensaje de aviso</b>'))
		self.et_aviso.set_use_markup(True)
		self.panel.put(self.et_aviso, 160, 65)

                # Mensaje adaptado
		self.mensaje = Gtk.Label()
		if msg != None:
			self.mensaje.set_text("%s" % self.cadena)
		self.panel.put(self.mensaje, 30, 100)


		# Botonera de la ventana
		# El mensaje es dinamico, tamano de la ventana y de la colocacacion de sus elementos
		# depende del numero de lineas del mensaje, indicado por la variable self.indice_linea
		self.boton = Gtk.Button(_('Aceptar'))
		self.boton.connect("clicked", self.hide_event, None)
		self.boton.set_size_request(90,30)
		altura_bot = 150 + (12*self.indice_linea)
		self.panel.put(self.boton, 390, altura_bot - 10)


		# Dibujar la ventana de aviso
		altura = 190 + (12*self.indice_linea)
		self.warning_window.set_size_request(500, altura)
		self.imagen_aviso.show()
		self.et_aviso.show()
		self.mensaje.show()
		self.boton.show()
		self.panel.show()
		self.vbox.show()
		self.warning_window.show()

	def hide_event(self, widget, data = None):
		self.warning_window.hide()

	def delete_event(self, widget, data = None):
		self.warning_window.destroy()
		Gtk.main_quit()

	def destroy(self, widget, data = None):
		self.warning_window.destroy()
		Gtk.main_quit()

	def preparar_mensaje(self):
		num_car = len(self.cadena)
		act_car = 69  # numero de caracteres admitidos por cada linea
		num_lin = 1
		while act_car <= num_car:
			while self.cadena[act_car] != ' ': # cada 69 caractreres del mensaje se introduce un salto de linea
				act_car = act_car - 1
			self.cadena = self.cadena[:act_car] + '\n' + self.cadena[act_car+1:]
			num_lin = num_lin + 1
			act_car = act_car + 70
		return num_lin

class parda_Frontend:
	def __init__(self):
		self.tam_home = -1
		self.window = Gtk.Window(Gtk.WindowType.TOPLEVEL)
		self.window.set_position(Gtk.WindowPosition.CENTER)
		self.window.set_border_width(10)
		self.window.set_size_request(660, 500)
		self.window.set_title(_('Parda'))
		self.window.set_wmclass(_('Parda'), _('Parda'))
		self.window.set_icon_from_file('%s/img/parda_icon.png' % PATH)

		self.box = Gtk.VBox(False, 10)
		self.window.add(self.box)

		self.window.connect("delete_event", self.delete_event)
		self.window.connect("destroy", self.destroy)

		# Lista de dispositivos
		# - Etiqueta de la lista
		self.et_lista = Gtk.Label()
		self.et_lista.set_markup(_('<b>Dispositivos Conectados</b>'))
		self.et_lista.set_use_markup(True)
		self.box.pack_start(self.et_lista, False, True, 0)
		# - Lista
		self.contenedor_lista = Gtk.ScrolledWindow()
		self.listado = Gtk.ListStore(str, str, str, str)
		self.vista_arbol_lista = Gtk.TreeView()
		self.vista_arbol_lista.set_model(self.listado)
		self.contenido_columna_1 = Gtk.CellRendererText()
		self.columna_1 = Gtk.TreeViewColumn(_('Dispositivo'), self.contenido_columna_1, text=0)
		self.vista_arbol_lista.append_column(self.columna_1)
		self.contenido_columna_2 = Gtk.CellRendererText()
		self.columna_2 = Gtk.TreeViewColumn(_('Dev'), self.contenido_columna_2, text=1)
		self.columna_2.set_visible(False)
		self.vista_arbol_lista.append_column(self.columna_2)
		self.contenido_columna_3 = Gtk.CellRendererText()
		self.columna_3 = Gtk.TreeViewColumn(_('Capacidad'), self.contenido_columna_3, text=2)
		self.vista_arbol_lista.append_column(self.columna_3)
		self.contenido_columna_4 = Gtk.CellRendererText()
		self.columna_4 = Gtk.TreeViewColumn(_('Etiqueta'), self.contenido_columna_4, text=3)
		self.vista_arbol_lista.append_column(self.columna_4)
		self.seleccion_vista = self.vista_arbol_lista.get_selection()
		self.seleccion_vista.connect("changed", self.actualizar_info_dispositivo)
		self.contenedor_lista.add(self.vista_arbol_lista)
		self.contenedor_lista.show_all()
		self.box.pack_start(self.contenedor_lista, True, True, 0)
		# Fin lista de dispositivos
	
		# Cuadro informativo
		# -Cabecera
		self.hbox_info_cab = Gtk.HBox(False, 5)
		self.et_cab_info = Gtk.Label(_('Dispositivo selecionado: '))
		self.et_cab_info_dev = Gtk.Label()
		self.et_cab_info_dev.set_markup(_('<i>dispositivo no seleccionado</i>'))
		self.et_cab_info_dev.set_use_markup(True)
		self.hbox_info_cab.pack_start(self.et_cab_info, False, True, 0)
		self.hbox_info_cab.pack_start(self.et_cab_info_dev, False, True, 0)
		self.et_cab_info.show()
		self.et_cab_info_dev.show()
		self.hbox_info_cab.show()
		self.box.pack_start(self.hbox_info_cab, False, True, 0)
		# -Barra de espacio libre
		self.espacio = Gtk.ProgressBar()
		self.box.pack_start(self.espacio, False, True, 0)
		self.espacio.set_text(_('desconocido'))
		self.espacio.set_show_text(True)
		self.espacio.show()
		# -Datos del dispositivo
		self.hbox_info_cap = Gtk.HBox(False, 5)
		self.et_info_cap = Gtk.Label(_('Capacidad: '))
		self.info_cap = Gtk.Label()
		self.info_cap.set_markup(_('<i>desconocido</i>'))
		self.info_cap.set_use_markup(True)
		self.hbox_info_cap.pack_start(self.et_info_cap, False, True, 0)
		self.hbox_info_cap.pack_start(self.info_cap, False, True, 0)
		self.et_info_cap.show()
		self.info_cap.show()
		self.hbox_info_cap.show()
		self.box.pack_start(self.hbox_info_cap, False, True, 0)
				
		self.hbox_info_lib = Gtk.HBox(False, 5)
		self.et_info_lib = Gtk.Label(_('Espacio libre: '))
		self.info_lib = Gtk.Label()
		self.info_lib.set_markup(_('<i>desconocido</i>'))
		self.info_lib.set_use_markup(True)
		self.hbox_info_lib.pack_start(self.et_info_lib, False, True, 0)
		self.hbox_info_lib.pack_start(self.info_lib, False, True, 0)
		self.et_info_lib.show()
		self.info_lib.show()
		self.hbox_info_lib.show()
		self.box.pack_start(self.hbox_info_lib, False, True, 0)
				
		self.hbox_info_pm = Gtk.HBox(False, 5)
		self.et_info_pm = Gtk.Label(_('Punto de montaje: '))
		self.info_pm = Gtk.Label()
		self.info_pm.set_markup(_('<i>desconocido</i>'))
		self.info_pm.set_use_markup(True)
		self.hbox_info_pm.pack_start(self.et_info_pm, False, True, 0)
		self.hbox_info_pm.pack_start(self.info_pm, False, True, 0)
		self.et_info_pm.show()
		self.info_pm.show()
		self.hbox_info_pm.show()
		self.box.pack_start(self.hbox_info_pm, False, True, 0)
		
		self.hbox_info_home = Gtk.HBox(False, 5)
		self.et_info_home = Gtk.Label(_('Espacio ocupado en el directorio personal: '))
		self.info_home = Gtk.Label()
		self.info_home.set_markup(_('<i>Calculando</i>'))
		self.info_home.set_use_markup(True)
		self.hbox_info_home.pack_start(self.et_info_home, False, True, 0)
		self.hbox_info_home.pack_start(self.info_home, False, True, 0)
		self.et_info_home.show()
		self.info_home.show()
		self.hbox_info_home.show()
		self.box.pack_start(self.hbox_info_home, False, True, 0)
		hilo_calculo = threading.Thread(target=self.actualizar_tamano_home)
		hilo_calculo.daemon = True
		hilo_calculo.start()

		# -Boton de copia
		self.hbox_boton_cop = Gtk.HBox(False,5)
		self.b_iniciar_cop = Gtk.Button()
		self.b_iniciar_cop.set_tooltip_text(_('Realizar copia de seguridad del directorio personal en un dispositivo USB externo'))
		self.imagen_backup = Gtk.Image()
		self.imagen_backup.set_from_file('%s/img/backup.png' % PATH)
		self.b_iniciar_cop.set_image(self.imagen_backup)
		self.b_iniciar_cop.connect("clicked", self.informacion_copia_seguridad)
		self.b_iniciar_cop.set_sensitive(False)
		self.hbox_boton_cop.pack_start(self.b_iniciar_cop, True, False, 0)
		self.b_iniciar_cop.show()
		self.b_restaurar_cop = Gtk.Button()
		self.b_restaurar_cop.set_tooltip_text(_('Restaurar copia de seguridad desde un dispositivo USB externo'))
		self.imagen_restore = Gtk.Image()
		self.imagen_restore.set_from_file('%s/img/restore.png' % PATH)
		self.b_restaurar_cop.set_image(self.imagen_restore)
		self.b_restaurar_cop.connect("clicked", self.informacion_restaurar_copia)
		self.b_restaurar_cop.set_sensitive(False)
		self.hbox_boton_cop.pack_start(self.b_restaurar_cop, True, False, 0)
		self.b_restaurar_cop.show()
		self.hbox_boton_cop.show()
		self.box.pack_start(self.hbox_boton_cop, False, False, 0)
		# Fin de cuadro informativo

		# Bloque de progreso de operaciones
		self.box_pro = Gtk.VBox(False, 10)
		# Informacion progreso
		self.et_operacion = Gtk.Label("")
		self.box_pro.pack_start(self.et_operacion, False, True, 0)
		self.hbox_info_pro = Gtk.HBox(False, 5)
		self.icon_exito = Gtk.Image()
		self.icon_exito.set_from_file('%s/img/exito.png' % PATH)
		self.box_pro.pack_start(self.icon_exito, False, True, 0)
		self.et_operacion.show()
		self.etiqueta_progreso = Gtk.Label("")
		self.dato_progreso = Gtk.Label("")
		self.hbox_info_pro.pack_start(self.etiqueta_progreso, False, True, 0)
		self.hbox_info_pro.pack_start(self.dato_progreso, False, True, 0)
		self.hbox_info_pro.show()
		self.box_pro.pack_start(self.hbox_info_pro, False, True, 5)
		self.etiqueta_progreso.show()
		self.dato_progreso.show()
		self.progreso = Gtk.ProgressBar()
		self.box_pro.pack_start(self.progreso, False, True, 0)
		self.progreso.show()
		# Resumen de operacion de copia
		self.et_resumen_pro = Gtk.Label("")
		self.box_pro.pack_start(self.et_resumen_pro, False, True, 0)
		self.box.pack_start(self.box_pro, False, False, 0)
		# Fin bloque de progreso de operaciones
	
		self.backend = parda_Backend(self)
		self.backend.detectar_dispositivos()

		self.et_lista.show()
		self.box.show()	
		self.window.show()
	
	def actualizar_tamano_home(self):
		self.tam_home = calcular_tamano(os.getenv('HOME'))
		GObject.idle_add(self.info_home.set_markup, format_size(self.tam_home))

	def actualizar_dispositivos(self, devices):
		self.dispositivos = {}
		self.listado.clear()
		for device in devices:
			self.dispositivos[devices[device]['device']] = devices[device]
			self.incluir_dispositivo(devices[device])

	def incluir_dispositivo(self, disp):
		texto = disp['vendor'] + " " + disp['model'] + " (" + disp['device'] + ")"
		self.listado.append([texto, disp['device'], format_size(disp['capacity']), disp['label']])

	def actualizar_info_dispositivo(self, widget, data = None):
		device = None
		(aux, iter) = self.seleccion_vista.get_selected()
		if iter is not None:
			device = aux.get_value(iter, 1)
		else:
			self.espacio.set_fraction(0.0)
			self.et_cab_info_dev.set_markup(_('<i>dispositivo no seleccionado</i>'))
			self.espacio.set_text(_('desconocido'))
			self.info_cap.set_markup(_('<i>desconocido</i>'))
			self.info_lib.set_markup(_('<i>desconocido</i>'))
			self.info_pm.set_markup(_('<i>desconocido</i>'))
			self.b_iniciar_cop.set_sensitive(False)
			self.b_restaurar_cop.set_sensitive(False)
			
		if device is not None:
			try:
				disp = self.dispositivos[device]
				fraccion = (disp['free'] * 100) / disp['capacity']
				self.espacio.set_fraction(1.0 - fraccion/100.0)
				self.espacio.set_text("%d" % fraccion+_('% libre'))
				self.et_cab_info_dev.set_markup(aux.get_value(iter, 0))
				self.info_cap.set_markup(format_size(disp['capacity']))
				if disp['mountpoint'] == None:
					(mountpoint, free) = self.backend.get_device_info(disp['obj'], disp['path'])
				else:
					free = disp['free']
					mountpoint = disp['mountpoint']
				if not self.espacio_libre_suficiente(disp):
					cad_free = '<span foreground="red">%s (Espacio insuficiente en el dispositivo USB)</span>' % format_size(free)
				else:
					cad_free = '<span foreground="green">%s</span>' % format_size(free)
				self.info_lib.set_markup(cad_free)
				self.info_pm.set_markup(mountpoint)
				self.b_iniciar_cop.set_sensitive(True)
				self.b_restaurar_cop.set_sensitive(True)
				self.dispositivo_seleccionado = disp
			except (KeyError) as e:
				pass
		
	def espacio_libre_suficiente(self, dispositivo):
		tamano_home = self.tam_home
		if tamano_home == -1:
			return False
		else:
			dircopia = '%s/copia_seguridad_parda/%s' % (dispositivo['mountpoint'], os.path.basename(os.environ.get('HOME')))
			tamano_dircopia = calcular_tamano(dircopia)
			if os.access(dircopia, os.F_OK) == False:
				esp_disp = dispositivo['free']
				esp_ocup = tamano_home
			else:
				esp_disp = dispositivo['free'] + tamano_dircopia
				esp_ocup = tamano_home
			if esp_disp < esp_ocup:
				return False
			else:
				return True
			

	def comenzar_copia_seguridad(self, widget, data = None):
		self.window_info_cop.destroy()
		self.hbox_boton_cop.set_sensitive(False)
		self.vista_arbol_lista.set_sensitive(False)
		self.window.set_deletable(False)
		self.window.set_sensitive(True)
		if not self.espacio_libre_suficiente(self.dispositivo_seleccionado):
			self.window.set_sensitive(True)
			self.hbox_boton_cop.set_sensitive(True)
			self.vista_arbol_lista.set_sensitive(True)
			self.window.set_deletable(True)
			aviso = mensaje_Aviso(_('El dispositivo seleccionado no dispone de espacion suficiente para realizar la copia de seguridad de su directorio personal'))
		else:
			try:
				self.hbox_boton_cop.set_sensitive(False)
				self.vista_arbol_lista.set_sensitive(False)
				self.window.set_deletable(False)
				self.window.set_sensitive(True)
				hilo_copia=threading.Thread(target=self.copia_completa,args=(os.getenv('HOME'), '%s/copia_seguridad_parda' % self.dispositivo_seleccionado['mountpoint'], '1',))
				hilo_copia.daemon = True
				hilo_copia.start()
			except (Exception) as e:
				pass
		

	def comenzar_restore_backup(self, widget, data = None):
		self.window_info_rest.destroy()
		self.hbox_boton_cop.set_sensitive(False)
		self.vista_arbol_lista.set_sensitive(False)
		self.window.set_deletable(False)
		self.window.set_sensitive(True)
		try:
			hilo_restore = threading.Thread(target=self.copia_completa, args=('%s/copia_seguridad_parda/%s' % (self.dispositivo_seleccionado['mountpoint'], os.path.basename(os.environ.get('HOME'))), os.getenv('HOME'), '2',))
			hilo_restore.daemon = True
			hilo_restore.start()
		except (Exception) as e:
			pass
	
	def copia_completa(self, dirorigen, dirdestino, tipo):
		GObject.idle_add(self.progreso_operacion_activar, self.tam_home, tipo)
		diruser = os.path.basename(dirorigen)
		dirorigen += os.sep
		if tipo == '1':
			if self.borrar_anterior == True:
				GObject.idle_add(self.progreso_operacion_borrado)
				dircopia_anterior = '%s/copia_seguridad_parda/%s' % (self.dispositivo_seleccionado['mountpoint'], os.path.basename(os.environ.get('HOME')))
				shutil.rmtree(dircopia_anterior)
		for dir, subdirs, files in os.walk(dirorigen):
			if os.path.isabs(dirdestino):
				relpath = dir.replace(dirorigen,'')
				if re.search("^\.", relpath) != None:
					continue
				if tipo == '2':
					backup_dir = os.path.join(dirdestino, relpath)
				else:
					backup_dir = os.path.join(dirdestino, diruser, relpath)
			else:
				backup_dir = os.path.join(dir, dirdestino)
			if not os.path.exists(backup_dir):
				os.makedirs(backup_dir)

			for f in files:
				if re.search("^\.", f) != None:
					continue

				filepath = os.path.join(dir, f)
				destpath = os.path.join(backup_dir, f)
				if os.path.isfile(filepath):
					GObject.idle_add(self.progreso_operacion_actdatos, f)
				try:
					if os.path.isfile(destpath) and filecmp.cmp(destpath, filepath, shallow=False):
						GObject.idle_add(self.progreso_operacion_actfraccion, filepath)
						time.sleep(0.015)
						continue
				except (OSError) as e:
					pass
				try:
					shutil.copy2(filepath, destpath)
					GObject.idle_add(self.progreso_operacion_actfraccion, filepath)
				except (Exception) as e:
					pass

		GObject.idle_add(self.progreso_operacion_terminar)
		GObject.idle_add(self.b_iniciar_cop.set_sensitive, True)
		GObject.idle_add(self.b_restaurar_cop.set_sensitive, True)
		GObject.idle_add(self.window.set_deletable, True)

	def progreso_operacion_activar(self, total, tipo):
		if tipo == '1':
                        titulo = _('Realizando copia de seguridad')
                        leyenda = _('Copiando: ')
                        resumen = _('Copia de seguridad completada correctamente')
		elif tipo == '2':
			titulo = _('Restaurando copia de seguridad')
			leyenda = _('Copiando: ')
			resumen = _('Copia de seguridad restaurada correctamente')
		elif tipo == '3':
			titulo = _('Calculando tamano del directorio personal')
			leyenda = _('Comprobando')
			resumen = ''
		elif tipo == '4':
			titulo = _('Actualizando copia de seguridad')
			leyenda = _('Actualizando')
			resumen = _('Copia de seguridad actualizada correctamente')
		else:
			titulo = _('Operacion desconocida')
			leyenda = _('Desconocido :')
			resumen = ''
		
		self.total_progreso=total
		self.acumulado_progreso=0
	
		self.icon_exito.hide()
		self.hbox_info_pro.show()
		self.et_operacion.show()
		self.et_operacion.set_label(titulo)
		self.etiqueta_progreso.set_label(leyenda)
		self.et_resumen_pro.hide()
		self.et_resumen_pro.set_label(resumen)
		self.box_pro.show()
		
	def progreso_operacion_terminar(self):
		self.hbox_info_pro.hide()
		self.progreso.set_text(_('100% Completado'))
		self.progreso.set_fraction(1)
		self.et_operacion.hide()
		self.et_resumen_pro.show()
		self.icon_exito.show()
		self.hbox_boton_cop.set_sensitive(True)
		self.vista_arbol_lista.set_sensitive(True)
		
	def progreso_operacion_borrado(self):
		self.dato_progreso.set_text(_('Borrando copia de seguridad previa'))

	def progreso_operacion_actdatos(self, nombre_rel):
		time.sleep(0.1)
		self.dato_progreso.set_label(nombre_rel)

	def progreso_operacion_actfraccion(self, nombre_abs):
		self.acumulado_progreso = self.acumulado_progreso + getsize(nombre_abs)
		if self.acumulado_progreso >= self.total_progreso:
        		fraccion = 100
		else:
                       	fraccion = (self.acumulado_progreso * 100) / self.total_progreso
		self.progreso.set_fraction(fraccion/100.0)
		self.progreso.set_text("%d" % fraccion+_('% Completado'))
		self.progreso.set_show_text(True)

	def informacion_restaurar_copia(self, widget, data = None):
		dircopia = '%s/copia_seguridad_parda/%s' % (self.dispositivo_seleccionado['mountpoint'], os.path.basename(os.environ.get('HOME')))
		nom_user = os.environ.get('USER')
		if  os.access(dircopia, os.F_OK) == False:
			aviso = mensaje_Aviso(_('No se han encontrado copias de seguridad en el dispositivo externo seleccionado para el usuario que utiliza en estos momentos'))
			return 1

		self.window.set_sensitive(False)
		self.window_info_rest = Gtk.Window()
		self.window_info_rest.set_position(Gtk.WindowPosition.CENTER)
		self.window_info_rest.set_icon_from_file('%s/img/info.png' % PATH)
		self.window_info_rest.set_title(_('Restaurar copia de seguridad'))
		self.window_info_rest.connect("delete_event", self.cancelar_restauracion_copia)
		vbox_info = Gtk.VBox(False, 0)
		self.window_info_rest.add(vbox_info)
		hbox_mensaje = Gtk.HBox(False, 0)
		vbox_info.pack_start(hbox_mensaje, True, True, 20)
		vbox_info.show()
		hbox_mensaje.show()
		
		mensaje = Gtk.Label(_('Se va a proceder a recuperar la copia de seguridad de los datos\ndel usuario "%s" almacenados en el dispositivo externo.\nTenga en cuenta que los datos que sean diferentes\nde su directorio personal van a ser sustituidos por los que\nse encuentran almacenados en la copia de seguridad.') % nom_user)
		hbox_mensaje.pack_start(mensaje, False, True, 20)
		mensaje.show()

		hbox_botones = Gtk.HBox(True, 5)
		vbox_info.pack_start(hbox_botones, False, True, 10)
		hbox_botones.show()
		boton_cancelar = Gtk.Button(_('Cancelar'))
		imagen_cancelar = Gtk.Image()
		imagen_cancelar.set_from_file('%s/img/cancelar.png' % PATH)
		boton_cancelar.set_image(imagen_cancelar)
		boton_cancelar.connect('clicked', self.cancelar_restauracion_copia)
		boton_siguiente = Gtk.Button(_('Siguiente'))
		imagen_siguiente = Gtk.Image()
		imagen_siguiente.set_from_file('%s/img/siguiente.png' % PATH)
		boton_siguiente.set_image(imagen_siguiente)
		boton_siguiente.connect('clicked', self.comenzar_restore_backup, dircopia)
		hbox_botones.pack_start(boton_cancelar, True, False, 10)
		hbox_botones.pack_start(boton_siguiente, True, False, 10)
		boton_cancelar.show()
		boton_siguiente.show()
		self.window_info_rest.show()

	def informacion_copia_seguridad(self, widget, data = None):
		if self.dispositivo_seleccionado['fstype'] == 'vfat':
			dircopia = '%s/copia_seguridad_parda/%s' % (self.dispositivo_seleccionado['mountpoint'], os.path.basename(os.environ.get('HOME')))
			nom_user = os.environ.get('USER')
			if os.access(dircopia, os.F_OK) == False:
				# Esto no funciona, hay que hacer algo para cuando no esta creado el directorio de copia en el destino
				mensaje = Gtk.Label(_('En el dispositivo seleccionado no existe una copia de seguridad previa del usuario "%s".\nSe va a realizar una copia de seguridad de sus datos.') % nom_user)
			else:
				mensaje = Gtk.Label(_('Se ha encontrado en el dispositivo externo seleccionado una copia de seguridad previa del usuario "%s".\nSeleccione una de las opciones siguientes:') % nom_user)
		
			self.borrar_anterior = False
			self.window.set_sensitive(False)
			self.window_info_cop = Gtk.Window()
			self.window_info_cop.set_position(Gtk.WindowPosition.CENTER)
			self.window_info_cop.set_icon_from_file('%s/img/info.png' % PATH)
			self.window_info_cop.set_title(_('Crear copia de seguridad'))
			self.window_info_cop.connect("delete_event", self.cancelar_copia_seguridad)
			vbox_info = Gtk.VBox(False, 0)
			self.window_info_cop.add(vbox_info)
			hbox_mensaje = Gtk.HBox(False, 0)
			vbox_info.pack_start(hbox_mensaje, True, True, 20)
			vbox_info.show()
			hbox_mensaje.show()
		
			hbox_mensaje.pack_start(mensaje, False, True, 20)
			mensaje.show()

			if os.access(dircopia, os.F_OK) != False:
				hbox_rad_boton = Gtk.HBox(False, 0)
				vbox_rad_boton = Gtk.VBox(False, 5)
				vbox_info.pack_start(hbox_rad_boton, True, True, 0)
				hbox_rad_boton.pack_start(vbox_rad_boton, True, True, 20)
				vbox_rad_boton.show()
				hbox_rad_boton.show()

				rad_boton = Gtk.RadioButton.new_with_label_from_widget(None, _('Borrar la copia de seguridad previa y crearla de nuevo\n(Recomendado - tiempo de copia mayor)'))
				rad_boton.connect("toggled", self.seleccion_borrar, True)
				vbox_rad_boton.pack_start(rad_boton, True, True, 0)
				rad_boton.show()

				rad_boton = Gtk.RadioButton.new_with_label_from_widget(rad_boton, _('Actualizar la copia de seguridad con los datos actualizados de su directorio personal\n(Menor tiempo de copia, no se borran ficheros obsoletos)'))
				rad_boton.connect("toggled", self.seleccion_borrar, False)
				rad_boton.set_active(True)
				vbox_rad_boton.pack_start(rad_boton, True, True, 0)
				rad_boton.show()

			hbox_botones = Gtk.HBox(True, 5)
			vbox_info.pack_start(hbox_botones, False, True, 10)
			hbox_botones.show()
			boton_cancelar = Gtk.Button(_('Cancelar'))
			boton_cancelar.set_always_show_image(True)
			imagen_cancelar = Gtk.Image()
			imagen_cancelar.set_from_file('%s/img/cancelar.png' % PATH)
			boton_cancelar.set_image(imagen_cancelar)
			boton_cancelar.connect('clicked', self.cancelar_copia_seguridad)
			boton_siguiente = Gtk.Button(_('Siguiente'))
			boton_siguiente.set_always_show_image(True)
			imagen_siguiente = Gtk.Image()
			imagen_siguiente.set_from_file('%s/img/siguiente.png' % PATH)
			imagen_siguiente.set_pixel_size(12)
			boton_siguiente.set_image(imagen_siguiente)
			boton_siguiente.connect('clicked', self.comenzar_copia_seguridad, None)
			hbox_botones.pack_start(boton_cancelar, True, False, 10)
			hbox_botones.pack_start(boton_siguiente, True, False, 10)
			boton_cancelar.show()
			boton_siguiente.show()
			self.window_info_cop.show()
		else:
			aviso = mensaje_Aviso(_('Se ha detectado que su dispositivo tiene un formato de ficheros incompatible con la aplicación, para poder usarla deberá formatear su dispositivo con formato FAT32'))


	def seleccion_borrar(self, widget, data = None):
		if widget.get_active():
			self.borrar_anterior = data

	def cancelar_copia_seguridad(self, widget, data = None):
		self.window_info_cop.destroy()
		self.window.set_sensitive(True)

	def cancelar_restauracion_copia(self, widget, data = None):
		self.window_info_rest.destroy()
		self.window.set_sensitive(True)

	def delete_event(self, widget, data = None):
		Gtk.main_quit()

	def destroy(self, widget, data = None):
		Gtk.main_quit()

def main():
	Gtk.main()

if __name__ =="__main__":
	front = parda_Frontend()
	main()
